# Common functions used by rake tasks

# This is a wrapper for the standard rake task that adds some more features:
# - when the task ends (regardless if successfully or with an error),
#   it is reenabled (ready to run again within the same rake run)
#   unless 'allow_repeating_tasks' environment variable is set to false
# - the task that is created is enhanced with methods get_parameter and should_fail_on_exists
def repeatable_task(name, options = {}, &block)

  # Create a standard rake task that's reenabled (depending on environment settings) once it ends
  taskobj = task name, options do |task, args|
    begin
      task.set_args(args)
      block.call task
    ensure
      task.maybe_reenable
    end
  end

  # Enhance the task with a method to retrieve a parameter.
  # If the parameter is not defined as a task argument and an environment variable with the specified name
  # exists then the variable is used, however, with a deprecation warning. Such a warning is printed out
  # only once per rake run to reduce output noise.
  def taskobj.get_parameter(name)
    unless value = @args[name]
      value = ENV[name.to_s]

      if value
        if ! @already_warned
          $stderr.puts 'Warning: Passing rake task parameters as environment variables is deprecated, use task arguments instead, e.g. "rake some:task[param1,param2]".'
          @already_warned = true
        end
      end
    end

    value
  end

  # Enhance the task with a method to decide whether an attempt to create an already existing object
  # should end with a failure or if creation of such an object should simply be skipped.
  # The behaviour is dependent on the value of the 'onexists' parameter:
  # - fail => the task should fail with an unrecoverable error
  # - skip => the task should skip creation of the object and allow other tasks to continue the rake run
  def taskobj.should_fail_on_exists
    @args.with_defaults :onexists => 'fail'

    option = get_parameter(:onexists)

    case option
      when 'skip'
        return false
      when 'fail'
        return true
      else
        puts "Invalid value for 'onexists': '#{option}'. Use 'skip' or 'fail'."
        exit 1
    end
  end

  # This is a helper method to store arguments in the task instance
  def taskobj.set_args(args)
    @args = args
  end

  # This method reenables the task unless environment variable 'allow_repeating_tasks' is set to false
  def taskobj.maybe_reenable
    reenable unless ENV['allow_repeating_tasks'] == 'false'
  end
end

def get_node(node_name)
  begin
    node = Node.find_by_name(node_name)
    unless node
      puts "Node #{node_name} doesn't exist!"
      exit 1
    end
  rescue NameError
    puts 'Must specify node name (name=<hostname>).'
    exit 1
  rescue => e
    puts "There was a problem finding the node: #{e.message}"
    exit 1
  end
  node
end

def get_group(group_name)
  group = NodeGroup.find_by_name(group_name)
  unless group
    puts "Node group #{group_name} doesn't exist!"
    exit 1
  end
  group
rescue NameError
  puts 'Must specify node group name (name=<groupname>).'
  exit 1
rescue => e
  puts "There was a problem finding the node group: #{e.message}"
  exit 1
end

def get_class(class_name)
  nodeclass = NodeClass.find_by_name(class_name)
  unless nodeclass
    puts "Node class #{class_name} doesn't exist!"
    exit 1
  end
  nodeclass
rescue NameError
  puts 'Must specify node class name (name=<classname>).'
  exit 1
rescue => e
  puts "There was a problem finding the node class: #{e.message}"
  exit 1
end

